#  Copyright 2020, Olivier 'reivilibre'.
#
#  This file is part of Scone.
#
#  Scone is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Scone is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Scone.  If not, see <https://www.gnu.org/licenses/>.
from pathlib import Path
from typing import Any, Union

from jinja2 import DictLoader, Environment

from scone.default.steps.basic_steps import exec_no_fails
from scone.default.utensils.basic_utensils import WriteFile
from scone.head.kitchen import Kitchen


async def depend_remote_file(path: str, kitchen: Kitchen) -> None:
    # TODO not supported yet :(
    # sha256 = await kitchen.ut1(HashFile(path))
    # kitchen.get_dependency_tracker().register_remote_file(path, sha256)
    kitchen.get_dependency_tracker().register_remote_file(path)


def template_jinja2_builtin(path: Path, **template_vars: Any) -> str:
    template_text = path.read_text("UTF-8")

    try:
        env = Environment(
            loader=DictLoader({str(path): template_text}), autoescape=False
        )
        template = env.get_template(str(path))
        return template.render(template_vars)
    except Exception as e:
        raise RuntimeError(f"Error templating built-in: {path}") from e


async def write_sous_file(
    kitchen: Kitchen, path: str, mode: int, data: Union[str, bytes]
) -> None:
    if isinstance(data, str):
        data = data.encode("UTF-8")

    chan = await kitchen.start(WriteFile(path, mode))
    await chan.send(data)
    await chan.send(None)
    if await chan.recv() != "OK":
        raise RuntimeError(f"WriteFile failed to {path}")


async def delete_sous_file(kitchen: Kitchen, path: str):
    await exec_no_fails(kitchen, ["rm", path], "/")
